#!/bin/csh
# general installation script for lancelot on SUN worktation under SOLARIS

#
#  Ph. Toint, June 1996, for CGT Productions
#

#
#  basic system commands
#

set MKDIR=/bin/mkdir
set MAKE=/usr/ccs/bin/make

#
#  Fortran compiler
#

set FORTRAN=f77

#
#  Directory for temporary files
#

set TMP=/tmp

#
#  check flags
#

set err1 = 'ok'
switch ( $1 )
  case 'single':
  case 'SINGLE':
  case 'double':
  case 'DOUBLE':
       breaksw
  default:
       echo ' ** Error. First argument not single or double '
       set err1 = 'not'
       breaksw
endsw

set err2 = 'ok'
switch( $2 )
   case 'large':
   case 'LARGE':
   case 'medium':
   case 'MEDIUM':
   case 'small':
   case 'SMALL':
        breaksw
   default:
        echo ' ** Error. Second argument not large, medium or small '
        set err2 = 'not'
        breaksw
endsw

if( $err1 == 'not' || $err2 == 'not' )then
  exit 1
endif

#
#  compile the size casting program
#

if($2 == 'large' || $2 == 'LARGE')then
  cd ./lancelot/sources
  $FORTRAN -o tobig tobig.f
  echo ' '
  echo 'building large version of LANCELOT'
  echo ' '
else if($2 == 'medium' || $2 == 'MEDIUM')then
  cd ./lancelot/sources
  $FORTRAN -o tomed tomed.f
  echo ' '
  echo 'building medium version of LANCELOT'
  echo ' '
else if($2 == 'small' || $2 == 'SMALL')then
  cd ./lancelot/sources
  $FORTRAN -o totoy totoy.f
  echo ' '
  echo 'building small version of LANCELOT'
  echo ' '
endif

#
#  Single precision version
#

if( $1 == 'single' || $1 == 'SINGLE')then
  cd ../objects
  if(! -e single)$MKDIR single
  cd ../sources
  $FORTRAN -o tosngl tosngl.f
  cd ./sifdec
  echo ' '
  echo 'compiling single precision sif decoder'
  echo ' '
  if($2 == 'small'  || $2 == 'SMALL')  $MAKE sifdec_s "TMP=$TMP" "SIZE=totoy"
  if($2 == 'medium' || $2 == 'MEDIUM') $MAKE sifdec_s "TMP=$TMP" "SIZE=tomed"
  if($2 == 'large'  || $2 == 'LARGE')  $MAKE sifdec_s "TMP=$TMP" "SIZE=tobig"
  cd ../optimizers
  echo ' '
  echo 'compiling single precision lancelot optimizers'
  echo ' '
  if($2 == 'small'  || $2 == 'SMALL')  $MAKE lancelot_s "TMP=$TMP" "SIZE=totoy"
  if($2 == 'medium' || $2 == 'MEDIUM') $MAKE lancelot_s "TMP=$TMP" "SIZE=tomed"
  if($2 == 'large'  || $2 == 'LARGE')  $MAKE lancelot_s "TMP=$TMP" "SIZE=tobig"
  cd ../..

#
#  double precision version
#

else if( $1 == 'double' || $1 == 'DOUBLE')then
  cd ../objects
  if(! -e double)$MKDIR double
  cd ../sources
  $FORTRAN -o todble todble.f
  cd ./sifdec
  echo ' '
  echo 'compiling double precision sif decoder'
  echo ' '
  if($2 == 'small'  || $2 == 'SMALL')  $MAKE sifdec_d "TMP=$TMP" "SIZE=totoy"
  if($2 == 'medium' || $2 == 'MEDIUM') $MAKE sifdec_d "TMP=$TMP" "SIZE=tomed"
  if($2 == 'large'  || $2 == 'LARGE')  $MAKE sifdec_d "TMP=$TMP" "SIZE=tobig"
  cd ../optimizers
  echo ' '
  echo 'compiling double precision lancelot optimizers'
  echo ' '
  if($2 == 'small'  || $2 == 'SMALL')  $MAKE lancelot_d "TMP=$TMP" "SIZE=totoy"
  if($2 == 'medium' || $2 == 'MEDIUM') $MAKE lancelot_d "TMP=$TMP" "SIZE=tomed"
  if($2 == 'large'  || $2 == 'LARGE')  $MAKE lancelot_d "TMP=$TMP" "SIZE=tobig"
  cd ../..
endif

exit 0

