#!/bin/csh
# general unfolding script for lancelot on SUN workstation under Solaris

#
#  N. Gould and Ph. Toint, December 4th, 1990, for CGT Productions.
#

#
#  define basic system commands
#

set MKDIR=/bin/mkdir
set CP=/bin/cp
set CHMOD=/bin/chmod
set RM="/bin/rm -f"
set MV=/bin/mv

#
#  create the LANCELOT directory structure
#

if(! -e lancelot) $MKDIR lancelot
if(! -e sampleproblems) $MKDIR sampleproblems
cd lancelot
if(! -e sources) $MKDIR sources
cd sources
if(! -e sifdec) $MKDIR sifdec
if(! -e optimizers) $MKDIR optimizers
cd ..
if(! -e objects) $MKDIR objects
cd ..

#
#  transfer system dependent procedures
#

$CP todble.sol  ./lancelot/sources/todble.f
$CP tosngl.sol  ./lancelot/sources/tosngl.f

$CP makesd.sol  ./lancelot/sources/sifdec/makefile
$CP makeop.sol  ./lancelot/sources/optimizers/makefile

$CP sdlan.sol   ./lancelot/sdlan
$CP lan.sol     ./lancelot/lan
$CP compil.sol  ./lancelot/compil

$CP README.sol  READ.ME

#
#  make the shells executable
#

$CHMOD a+x ./lancelot/lan
$CHMOD a+x ./lancelot/sdlan

#
#  transfer code source files
#

$CP local.f     ./lancelot/sources/local.f

$CP decode.f    ./lancelot/sources/sifdec/decode.f
$CP gps.f       ./lancelot/sources/sifdec/gps.f
$CP inlanc.f    ./lancelot/sources/sifdec/inlanc.f
$CP makefn.f    ./lancelot/sources/sifdec/makefn.f
$CP makegr.f    ./lancelot/sources/sifdec/makegr.f
$CP printp.f    ./lancelot/sources/sifdec/printp.f
$CP rest.f      ./lancelot/sources/sifdec/rest.f
$CP runsd.f     ./lancelot/sources/sifdec/runsd.f
$CP utils.f     ./lancelot/sources/sifdec/utils.f

$CP asmbl.f     ./lancelot/sources/optimizers/asmbl.f
$CP assl.f      ./lancelot/sources/optimizers/assl.f
$CP auglg.f     ./lancelot/sources/optimizers/auglg.f
$CP dumbr.f     ./lancelot/sources/optimizers/dumbr.f
$CP bndsl.f     ./lancelot/sources/optimizers/bndsl.f
$CP cauch.f     ./lancelot/sources/optimizers/cauch.f
$CP cg.f        ./lancelot/sources/optimizers/cg.f
$CP datal.f     ./lancelot/sources/optimizers/datal.f
$CP drche.f     ./lancelot/sources/optimizers/drche.f
$CP drchg.f     ./lancelot/sources/optimizers/drchg.f
$CP dumhsl.f    ./lancelot/sources/optimizers/dumhsl.f
$CP elgrd.f     ./lancelot/sources/optimizers/elgrd.f
$CP frntl.f     ./lancelot/sources/optimizers/frntl.f
$CP gtpgr.f     ./lancelot/sources/optimizers/gtpgr.f
$CP hsprd.f     ./lancelot/sources/optimizers/hsprd.f
$CP initw.f     ./lancelot/sources/optimizers/initw.f
$CP inxac.f     ./lancelot/sources/optimizers/inxac.f
$CP lance.f     ./lancelot/sources/optimizers/lance.f
$CP linpac.f    ./lancelot/sources/optimizers/linpac.f
$CP mdchl.f     ./lancelot/sources/optimizers/mdchl.f
$CP misc.f      ./lancelot/sources/optimizers/misc.f
$CP others.f    ./lancelot/sources/optimizers/others.f
$CP precn.f     ./lancelot/sources/optimizers/precn.f
$CP runlan.f    ./lancelot/sources/optimizers/runlan.f
$CP sbmin.f     ./lancelot/sources/optimizers/sbmin.f
$CP scaln.f     ./lancelot/sources/optimizers/scaln.f
$CP sort.f      ./lancelot/sources/optimizers/sort.f
$CP speci.f     ./lancelot/sources/optimizers/speci.f

#
#  transfer sample problems
#

$CP ALLIN.SIF   ./sampleproblems/ALLIN.SIF
$CP HS65.SIF    ./sampleproblems/HS65.SIF
$CP SAMPLE.SIF  ./sampleproblems/SAMPLE.SIF

#
#  transfer other LANCELOT stuff
#

$CP SPEC.SPC    ./lancelot/SPEC.SPC
$CP SAMPLE.SPC  ./lancelot/SAMPLE.SPC

$CP tobig.f     ./lancelot/sources/tobig.f
$CP tomed.f     ./lancelot/sources/tomed.f
$CP totoy.f     ./lancelot/sources/totoy.f

#
#  clean up system dependent procedures
#

$RM todble.*
$RM tosngl.*

$RM makesd.*
$RM makeop.*

$RM sdlan.*
$RM lan.*
$RM compil.*

$RM README.*

#
#  clean up code source files
#

$RM local.f

$RM decode.f
$RM gps.f
$RM inlanc.f
$RM makefn.f
$RM makegr.f
$RM printp.f
$RM rest.f
$RM runsd.f
$RM utils.f

$RM asmbl.f
$RM assl.f
$RM auglg.f
$RM dumbr.f
$RM bndsl.f
$RM cauch.f
$RM cg.f
$RM datal.f
$RM drche.f
$RM drchg.f
$RM dumhsl.f
$RM elgrd.f
$RM frntl.f
$RM gtpgr.f
$RM hsprd.f
$RM initw.f
$RM inxac.f
$RM lance.f
$RM linpac.f
$RM mdchl.f
$RM misc.f
$RM others.f
$RM precn.f
$RM runlan.f
$RM sbmin.f
$RM scaln.f
$RM sort.f
$RM speci.f

#
#  clean up sample problems
#

$RM ALLIN.SIF HS65.SIF SAMPLE.SIF

#
#  clean up other LANCELOT stuff
#

$RM SPEC.SPC SAMPLE.SPC

$RM tobig.f tomed.f totoy.f

#

$CP instll.sol  fool.prf
$RM instll.*
$MV fool.prf  instll
$CHMOD a+x    instll
$RM unfold.*
